from manga_py.provider import Provider
from .helpers.std import Std


class MangaClubRu(Provider, Std):
    local_storage = None

    def get_archive_name(self) -> str:
        idx = self.get_chapter_index().split('-')
        return self.normal_arc_name({'vol': idx[0], 'ch': idx[1]})

    def get_chapter_index(self) -> str:
        idx = self.re.search(r'/manga/view/[^/]+/v(\d+)-c(\d+).html')
        return '{}-{}'.format(*idx.groups())

    def get_main_content(self):
        if not self.local_storage:
            self.get_manga_name()
        return self.http_get('{}/{}.html'.format(self.domain, self.local_storage[0]))

    def get_manga_name(self) -> str:
        selector = r'\.ru(?:/manga/view)?/(?:(\d+-.+)/(.+)\.html)'
        html = self.re.search(selector, self.get_url())
        self.local_storage = html.groups()
        return self.local_storage[1]

    def get_chapters(self):
        selector = '.manga-ch-list-item > a[href^="http"]'
        return self.document_fromstring(self.content, selector)

    def get_files(self):
        result = self.html_fromstring(self.chapter, '.manga-lines-page a.manga-lines')
        return [i.get('data-i') for i in result]

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaClubRu



from manga_py.provider import Provider
from .helpers.std import Std


class SenMangaCom(Provider, Std):

    def get_chapter_index(self) -> str:
        ch = self.chapter
        re = r'\.com/[^/]+/(\d+)([^/\d][^/]*)?/'
        idx = self.re.search(re, ch).groups()
        fmt = '{}'
        if idx[1]:
            fmt += '-{}'
        return fmt.format(*idx)

    def get_main_content(self):
        return self._get_content('{}/{}')

    def get_manga_name(self) -> str:
        return self._get_name(r'\.com/([^/]+)')

    def get_chapters(self):
        return self._elements('.list .element > .title > a')[::-1]

    def get_files(self):
        url = self.chapter
        parser = self.html_fromstring(url)
        pages = self._first_select_options(parser, 'select[name="page"]')
        src = parser.cssselect('#picture')[0].get('src')
        images = [src]
        for i in pages:
            images.append(self.re.sub(r'\d+\?token', i.get('value') + '?token', src))
        return images

    def prepare_cookies(self):
        self._base_cookies()

    def get_cover(self) -> str:
        return self._cover_from_content('.thumbnail > img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = SenMangaCom



from manga_py.provider import Provider
from .helpers.std import Std, Http2


class MangaFreakNet(Provider, Std):

    def get_archive_name(self):
        return self.chapter[0]

    def get_chapter_index(self) -> str:
        return self.re.search(r'.+_(\d+)', self.chapter[1]).group(1)

    def get_main_content(self):
        return self.http_get(self.get_url())

    def get_manga_name(self) -> str:
        return self._get_name('/Manga/([^?/#]+)')

    def get_chapters(self):
        items = self._elements('.manga_series_list td a[download]')
        return [(i.get('download'), i.get('href')) for i in items]

    def loop_chapters(self):
        items = self._storage['chapters'][::-1]
        Http2(self).download_archives([i[1] for i in items])

    def get_files(self):
        pass

    def prepare_cookies(self):
        self.cf_protect(self.get_url())

    def get_cover(self) -> str:
        return self._cover_from_content('.manga_series_image img')

    def book_meta(self) -> dict:
        # todo meta
        pass

    def chapter_for_json(self):
        return self.chapter[1]


main = MangaFreakNet



from manga_py.provider import Provider
from .helpers.std import Std


class MangaAe(Provider, Std):

    def get_chapter_index(self) -> str:
        return self.re.search(r'\.ae/[^/]+/(\d+)', self.chapter).group(1)

    def get_main_content(self):
        return self._get_content('{}/{}/')

    def get_manga_name(self) -> str:
        return self._get_name(r'\.ae/([^/]+)')

    def get_chapters(self):
        return self._elements('li > a.chapter')

    def get_files(self):
        img_selector = '#showchaptercontainer img'
        parser = self.html_fromstring(self.chapter)
        pages = parser.cssselect('#morepages a + a')
        images = self._images_helper(parser, img_selector)
        if pages:
            for i in pages:
                parser = self.html_fromstring(i.get('href'))
                images += self._images_helper(parser, img_selector)
        return images

    def get_cover(self) -> str:
        return self._cover_from_content('img.manga-cover')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaAe



from manga_py.provider import Provider
from .helpers.std import Std


class ComicsAndMangaRu(Provider, Std):

    def get_archive_name(self) -> str:
        index = self.get_chapter_index()
        return 'vol_{:0>3}'.format(index)

    def get_chapter_index(self) -> str:
        return self.re.search(r'.+/[^/]+?(\d+)$', self.chapter).group(1)

    def get_main_content(self):
        name = self.re.search('/(online-reading/[^/]+/[^/]+)', self.get_url())
        return self.http_get('{}/{}'.format(self.domain, name.group(1)))

    def get_manga_name(self):
        name = self.re.search('/online-reading/[^/]+/([^/]+)', self.get_url())
        return name.group(1)

    def get_chapters(self):
        selector = '.MagList > .MagListLine > a'
        items = self.document_fromstring(self.content, selector)
        return items[::-1]

    def get_files(self):
        img_selector = 'a > img'
        nu = self.http().normalize_uri
        uri = nu(self.chapter)
        parser = self.html_fromstring(uri, '.ForRead', 0)
        pages = parser.cssselect('.navigation select')[0].cssselect('option + option')
        images = self._images_helper(parser, img_selector)

        for i in pages:
            uri = '{}/{}'.format(nu(self.chapter.rstrip('/')), i.get('value'))
            parser = self.html_fromstring(uri, '.ForRead', 0)
            images += self._images_helper(parser, img_selector)

        return images

    def get_cover(self):
        pass  # TODO

    def book_meta(self) -> dict:
        # todo meta
        pass


main = ComicsAndMangaRu



from .gomanga_co import GoMangaCo


class HatigarmScansEu(GoMangaCo):
    _name_re = '/manga/([^/]+)'
    _content_str = '{}/manga/{}/'
    _chapters_selector = '.chapters [class^="chapter-title"] a'

    def get_chapter_index(self) -> str:
        url = self.chapter
        index_re = r'/manga/[^/]+/(\d+(?:\.\d+)?)'
        group = self.re.search(index_re, url).group(1)
        return group.replace('.', '-')

    def prepare_cookies(self):
        pass

    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        return self._images_helper(parser, '#all .img-responsive', 'data-src')


main = HatigarmScansEu



from manga_py.provider import Provider
from .helpers.std import Std


class LeitorNet(Provider, Std):
    __idx = None

    def get_archive_name(self) -> str:
        ch = self.chapter
        return self.normal_arc_name({
            'vol': ch['number'].split('.'),
            'ch': [ch['id_chapter'], ch['id_release']],
        })

    def get_chapter_index(self) -> str:
        return '-'.join(self.chapter['number'].split('.'))

    def get_main_content(self):
        idx = self.html_fromstring(self.get_url(), '[data-id-serie]', 0)
        self.__idx = idx.get('data-id-serie')
        return b'0'

    def get_manga_name(self) -> str:
        return self._get_name(r'/manga/([^/]+)')

    @staticmethod
    def __morph_chapters(items):
        result = []
        for item in items:  # items loop
            for r in item['releases']:  # items.releases loop
                release = item['releases'][r]
                result.append({
                    'number': item['number'],
                    'id_chapter': item['id_chapter'],
                    'id_serie': item['id_serie'],
                    'id_release': release['id_release'],
                    'link': release['link'],
                })
        return result

    def get_chapters(self):
        url = '{}/series/chapters_list.json?page={}&id_serie={}'
        items = []
        for i in range(1, 100):
            content = self.json.loads(self.http_get(url.format(
                self.domain, i, self.__idx
            ), headers={'x-requested-with': 'XMLHttpRequest'}))
            chapters = content.get('chapters', False)
            if not chapters:
                break
            items += chapters
        return self.__morph_chapters(items)

    def get_files(self):
        content = self.http_get(self.domain + self.chapter['link'])
        token = self.re.search(r'token=([^&]+)', content).group(1)
        url = '{}/leitor/pages.json?key={}&id_release={}'
        images = self.json.loads(self.http_get(url.format(
            self.domain,
            token, self.chapter['id_release']
        ), headers={'x-requested-with': 'XMLHttpRequest'}))
        print(images);exit()
        return images.get('images', {})

    def get_cover(self) -> str:
        url = '{}/manga/{}/{}'.format(
            self.domain,
            self.manga_name,
            self.__idx
        )
        image = self.html_fromstring(url, '.cover-image')
        if image and len(image):
            return self.parse_background(image[0])

    def book_meta(self) -> dict:
        pass

    def chapter_for_json(self):
        return self.domain + self.chapter['link']


main = LeitorNet



from manga_py.provider import Provider
from .helpers.std import Std


class LeoMangaCom(Provider, Std):

    def get_chapter_index(self) -> str:
        url = self.chapter
        idx = self.re.search(r'/manga/[^/]+/capitulo-(\d+)/([^/]+)/', url).groups()
        return '{1}-{0}'.format(*idx)

    def get_main_content(self):
        return self._get_content('{}/manga/{}')

    def get_manga_name(self) -> str:
        return self._get_name('/manga/([^/]+)')

    def _get_first_href(self, parser):
        n = self.http().normalize_uri
        url = n(parser[0].get('href'))
        select0 = self.html_fromstring(url, '.list-group .cap-option')
        if select0:
            return n(select0[0].get('href'))
        return None

    def get_chapters(self):
        chapter = self.document_fromstring(self.content, '.caps-list a')
        if chapter:
            url = self._get_first_href(chapter)
            if url:
                selector = '.viewcap-info select.form-control'
                parser = self.html_fromstring(url)
                options = self._first_select_options(parser, selector)
                return [i.get('value') for i in options[::-1]]
        return []

    def get_files(self):
        n = self.http().normalize_uri
        items = self.html_fromstring(self.chapter, '.vertical .cap-images')
        return [n(i.get('src')) for i in items]

    def get_cover(self) -> str:
        return self._cover_from_content('.cover img', 'data-original')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = LeoMangaCom



from manga_py.provider import Provider
from .helpers.jav_zip_org import JavZipOrg
from .helpers.std import Std


class ZipReadCom(Provider, Std):

    def get_chapter_index(self) -> str:
        idx = self.re.search(r'/.p=(\d+)', self.chapter).group(1)
        return '{}-{}'.format(self.chapter_id, idx)

    def get_main_content(self):
        pass

    def get_manga_name(self) -> str:
        return self._get_name(r'\.com/([^/]+)')

    def get_chapters(self):
        return self._elements('#content .entry > p > a')

    def get_files(self):
        jav_zip_org = JavZipOrg(self)
        return jav_zip_org.get_images()

    def get_cover(self):
        return self._cover_from_content('#content .entry p > img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = ZipReadCom



from manga_py.provider import Provider
from .helpers.std import Std


class ReadMngCom(Provider, Std):

    def get_chapter_index(self) -> str:
        ch = self.chapter
        re = r'\.com/[^/]+/(\d+(?:\.\d+)?)/?'
        idx = self.re.search(re, ch)
        if not idx:
            return self.re.search(r'\.com/[^/]+/([^/]+)', ch).group(1)
        return '-'.join(idx.group(1).split('.'))

    def get_main_content(self):
        return self._get_content('{}/{}')

    def get_manga_name(self) -> str:
        return self._get_name(r'\.com/([^/]+)')

    def get_chapters(self):
        return self._elements('.chp_lst li > a')

    def get_files(self):
        url = self.chapter + '/all-pages'
        parser = self.html_fromstring(url)
        return self._images_helper(parser, '.content-list img.img-responsive')

    def get_cover(self) -> str:
        return self._cover_from_content('.panel-body img.img-responsive')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = ReadMngCom



from manga_py.provider import Provider
from .helpers.std import Std


class PuzzmosCom(Provider, Std):

    def get_chapter_index(self) -> str:
        chapter = self.chapter
        idx = self.re.search('/manga/[^/]+/([^/]+)', chapter)
        return '-'.join(idx.group(1).split('.'))

    def get_main_content(self):
        return self._get_content('{}/manga/{}')

    def get_manga_name(self) -> str:
        return self._get_name('/manga/([^/]+)')

    def get_chapters(self):
        return self._elements('#bolumler td:first-child a')

    def get_files(self):
        img_selector = '.chapter-content img.chapter-img'
        url = self.chapter
        parser = self.html_fromstring(url)
        pages = parser.cssselect('.col-md-12 > .text-center > select option + option')
        images = self._images_helper(parser, img_selector)
        for i in pages:
            parser = self.html_fromstring(i.get('value'))
            images += self._images_helper(parser, img_selector)
        return images

    def get_cover(self) -> str:
        return self._cover_from_content('img.thumbnail.manga-cover')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = PuzzmosCom



from .zeroscans_com import ZeroScansCom


class TrashScanlationsCom(ZeroScansCom):
    def get_chapter_index(self) -> str:
        ch = self.chapter
        idx = self.re.search(self._chapter_selector, ch)
        idx = self.re.split(r'[^\d]', idx.group(1))
        return '-'.join(idx)

    def get_main_content(self):
        return self._get_content('{}/series/{}/')

    def get_manga_name(self) -> str:
        return self._get_name('/series/([^/]+)')


main = TrashScanlationsCom



import random

from manga_py.crypt import ManhuaGuiComCrypt
from manga_py.provider import Provider
from .helpers.std import Std


class ManhuaGuiCom(Provider, Std):
    servers = [
        'i.hamreus.com:8080',
        'us.hamreus.com:8080',
        'dx.hamreus.com:8080',
        'eu.hamreus.com:8080',
        'lt.hamreus.com:8080',
    ]

    def _get_ch_idx(self):
        chapter = self.chapter
        return self.re.search(r'/comic/[^/]+/(\d+)', chapter.get('href')).group(1)

    def get_archive_name(self) -> str:
        return super().get_archive_name() + '-' + self._get_ch_idx()

    def get_chapter_index(self) -> str:
        chapter = self.chapter
        span = chapter.cssselect('span')
        idx = self._get_ch_idx()
        if span:
            span = span[0].text_content()
            i = self.re.search(r'(\d+)', span).group(1)
            return '{}-{}'.format(i, idx)
        return '0-{}'.format(idx)

    def get_main_content(self):
        _ = self._get_name(r'/comic/(\d+)')
        return self.http_get('{}/comic/{}/'.format(self.domain, _))

    def get_manga_name(self) -> str:
        url = self.get_url()
        selector = 'h1'
        if self.re.search(r'/comic/\d+/\d+\.html', url):
            selector = 'h1 > a'
        return self.html_fromstring(url, selector, 0).text_content()

    def get_chapters(self):
        parser = self.document_fromstring(self.content)
        chapters = parser.cssselect('.chapter-list li > a')
        if not len(chapters):
            code = parser.cssselect('#__VIEWSTATE')[0].get('value')
            manhuagui = ManhuaGuiComCrypt()
            js = manhuagui.decrypt('LZString.decompressFromBase64("' + code + '")', '<a></a>')
            chapters = self.document_fromstring(js, '.chapter-list li > a')
        return chapters

    def parse_files_data(self, data):
        images = []
        md5 = data.get('sl', {}).get('md5', '')
        cid = data.get('cid', '')
        for i in data.get('files', []):
            prior = 3
            ln = len(self.servers)
            server = int(random.random() * (ln + prior))
            server = 0 if server < prior else server - prior
            images.append('http://{}{}{}?cid={}&md5={}'.format(
                self.servers[server],
                data.get('path', ''),
                i, cid, md5
            ))
        return images

    def get_files(self):
        url = self.chapter
        self._storage['referer'] = url
        content = self.http_get(url)
        js = self.re.search(r'\](\(function\(.+\))\s?<', content)
        if not js:
            return []
        manhuagui = ManhuaGuiComCrypt()
        data = manhuagui.decrypt(js.group(1), '')
        data = self.re.search(r'\(({.+})\)', data)
        if not data:
            return []
        data = self.json.loads(data.group(1))
        return self.parse_files_data(data)

    def get_cover(self):
        return self._cover_from_content('.hcover img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = ManhuaGuiCom



from .gomanga_co import GoMangaCo
from .helpers.std import Std


class RavensScansCom(GoMangaCo, Std):
    _name_re = '/(?:serie|read)/([^/]+)'
    __api_url = '/lector/api/v2/comic?stub='

    def get_main_content(self):
        url = '{}{}{}'.format(self.domain, self.__api_url, self.manga_name)
        return self.json.loads(self.http_get(url)).get('languages', [])

    def get_chapters(self):
        items = []
        for i in self.content:
            url = '{}{}{}&lang={}'.format(self.domain, self.__api_url, self.manga_name, i)
            items += self.json.loads(self.http_get(url)).get('chapters', [])
        return [i.get('href') for i in items[::-1]]  # DON'T TOUCH THIS!

    def get_cover(self) -> str:
        return self.content.get('fullsized_thumb_url', None)


main = RavensScansCom



from manga_py.provider import Provider
from .helpers.std import Std


class MerakiScansCom(Provider, Std):
    _name_re = 'com/([^/]+)'
    _content_str = '{}/{}/'

    def get_chapter_index(self) -> str:
        re = self.re.compile(self._name_re + '/([^/]+)')
        idx = re.search(self.chapter).group(2)
        return '-'.join(idx.split('.'))

    def _home_url(self):
        return self._content_str.format(self.domain, self.manga_name)

    def get_main_content(self):
        return self.http_get(self._home_url())

    def get_manga_name(self) -> str:
        return self._get_name(self._name_re)

    def _get_pages_count(self):
        items = self._elements('.pgg li:last-child > a')
        if items and len(items):
            href = items[0].get('href')
            re = self.re.compile(r'/chapter-list/(\d[^/]*)')
            return int(re.search(href).group(1))
        return 1

    def get_chapters(self):
        selector = 'ul.lst li > a'
        pages = self._get_pages_count()
        items = self._elements(selector)
        for i in range(1, pages):
            url = '{}/chapter-list/{}/'.format(self._home_url(), i + 1)
            content = self.http_get(url)
            items += self._elements(selector, content)
        return items

    def get_files(self):
        content = self.http_get(self.chapter)
        return self._images_helper(content, '#longWrap img')

    def get_cover(self) -> str:
        return self._cover_from_content('.mng_ifo img.cvr')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MerakiScansCom



from manga_py.provider import Provider
from .helpers.std import Std


class MangaKakalotCom(Provider, Std):

    def get_chapter_index(self) -> str:
        re = self.re.search('/chapter_([^/]+)', self.chapter)
        return re.group(1).replace('.', '-', 2)

    def get_main_content(self):
        return self._get_content('{}/manga/{}')

    def get_manga_name(self) -> str:
        return self._get_name('/(?:manga|chapter)/([^/]+)/?')

    def get_chapters(self):
        return self._elements('.chapter-list span a')

    def get_files(self):
        result = self.html_fromstring(self.chapter, '#vungdoc img')
        return [i.get('src') for i in result]

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaKakalotCom



from manga_py.provider import Provider
from .helpers.std import Std


class ReadComicBooksOnlineOrg(Provider, Std):
    _name_re = r'\.(?:org|net)/(?:reader/)?([^/]+)'

    def get_chapter_index(self) -> str:
        idx = self.re.search(r'/reader/[^/]+/[^/]+_(\d+(?:[\./]\d+)?)', self.chapter)
        if not idx:
            idx = self.re.search(r'/reader/[^/]+_(\d+(?:[\./]\d+)?)', self.chapter)
        return '-'.join(self.re.split(r'[/\.]', idx.group(1)))

    def get_main_content(self):
        if ~self.get_url().find('/reader/'):
            _url = self.html_fromstring(self.get_url(), 'td .verse a', 0).get('href')
            self._params['url'] = _url
        return self.http_get('{}/{}'.format(self.domain, self._get_name(self._name_re)))

    def get_manga_name(self) -> str:
        return self._get_name(self._name_re)

    def get_chapters(self):
        return self._elements('#chapterlist .chapter > a')

    def prepare_cookies(self):
        self._storage['domain_uri'] = self.domain.replace('/www.', '/')

    def _get_image(self, parser):
        src = parser.cssselect()
        if not src:
            return None
        return '{}/reader/{}'.format(self.domain, src[0].get('src'))

    def get_files(self):
        parser = self.html_fromstring(self.chapter + '?q=fullchapter')
        base = parser.cssselect('base')
        if base is not None and len(base):
            base = base[0].get('href')
        else:
            base = None
        n = self.http().normalize_uri
        return [n(i, base) for i in self._images_helper(parser, '#omv td > img')]

    def get_cover(self):
        self._cover_from_content(self.content, '.pic > img.series')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = ReadComicBooksOnlineOrg



from .hentaifox_com import HentaiFoxCom


class nHentaiNet(HentaiFoxCom):
    _idx_re = r'/g/(\d+)'
    _url_str = '{}/g/{}/'
    _name_selector = '#info h1'
    _cdn = 'https://i.nhentai.net/galleries/'
    __ext = {'j': 'jpg', 'p': 'png', 'g': 'gif'}

    def get_files(self):
        page = self._elements('#thumbnail-container a')[0]
        n = self.http().normalize_uri
        content = self.http_get(n(page.get('href')))
        imgs = self.re.search(r'gallery\s*:\s*(\{.+\}),', content)
        imgs = self.json.loads(imgs.group(1))
        idx = imgs.get('media_id')
        images = []
        for n, i in enumerate(imgs.get('images', {}).get('pages', [])):
            images.append('{}{}/{}.{}'.format(self._cdn, idx, n + 1, self.__ext.get(i.get('t'))))
        return images

    def get_cover(self) -> str:
        return self._cover_from_content('#cover img', 'data-src')


main = nHentaiNet



from .gomanga_co import GoMangaCo


class OtScansCom(GoMangaCo):
    _name_re = '/foolslide/[^/]+/([^/]+)/'
    _content_str = '{}/foolslide/series/{}/'
    _chapters_selector = '.list .group .element .title a'


main = OtScansCom



from manga_py.provider import Provider
from .helpers.std import Std


class MangaParkMe(Provider, Std):

    def get_chapter_index(self) -> str:
        selector = r'/manga/[^/]+/s.+?(?:/v(\d+))?/c(\d+[^/]*)'
        idx = self.re.search(selector, self.chapter).groups()
        if idx[0] is None:
            return '0-' + idx[1]
        return '-'.join(idx)

    def get_main_content(self):
        return self._get_content('{}/manga/{}')

    def get_manga_name(self) -> str:
        return self._get_name('/manga/([^/]+)')

    def get_chapters(self):
        return self._elements('div.stream:last-child em a:last-child')

    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        return self._images_helper(parser, '#viewer img.img')

    def get_cover(self):
        return self._cover_from_content('.cover img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaParkMe



from manga_py.provider import Provider
from .helpers.std import Std


class MangaBoxMe(Provider, Std):
    _local_storage = False

    def get_chapter_index(self) -> str:
        return self.re.search(r'/episodes/(\d+)', self.chapter).group(1)

    def get_main_content(self):
        if not self._local_storage:
            idx = self._get_name(r'/reader/(\d+)/episodes/')
            self._local_storage = True
            return self.http_get('{}/reader/{}/episodes/'.format(self.domain, idx))
        return self.content

    def get_manga_name(self) -> str:
        selector = 'meta[property="og:title"]'
        title = self.document_fromstring(self.content, selector, 0)
        return title.get('content').strip()

    def get_chapters(self):
        selector = '.episodes_list .episodes_item > a'
        return self._elements(selector)

    def get_files(self):
        items = self.html_fromstring(self.chapter, 'ul.slides li > img')
        return [i.get('src') for i in items]

    def get_cover(self):
        return self._cover_from_content('.episodes_img_main', 'data-src')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaBoxMe



from .taadd_com import TaaddCom


class TenMangaCom(TaaddCom):
    _name_selector = '.read-page a[href*="/book/"]'
    _pages_selector = '.sl-page'
    _chapters_selector = '.chapter-box .choose-page a:last-child'
    img_selector = '.pic_box .manga_pic'


main = TenMangaCom



from manga_py.provider import Provider


class AnimeXtremistCom:
    provider = None
    path = None

    def __init__(self, provider: Provider):
        self.provider = provider
        self.path = provider.get_manga_url()

    @staticmethod
    def build_path(item):
        return item[0] + item[1]

    @staticmethod
    def __sort(item, selector):
        _re = selector.search(item)
        if _re:
            return int(_re.group(1))
        return 0

    def sort_items(self, items):
        r = self.provider.re.compile(r'.+?-(\d+)')
        return sorted(items, key=lambda i: self.__sort(i[0], r))

    def sort_images(self, items):
        r = self.provider.re.compile(r'.+/.+-(\d+)[^/]*\.html')
        return sorted(items, key=lambda i: self.__sort(i, r))

    def _chapters(self, url=None):
        a = 'li + li > a'
        if url:
            items = self.provider.html_fromstring(url, a)
        else:
            items = self.provider.document_fromstring(self.provider.content, a)
        return items

    # http://animextremist.com/mangas-online/99love/
    def _chapters_with_dirs(self, items):
        result = []
        for i in items:
            href = i.get('href')
            url = '{}{}'.format(self.path, href)
            result += [(href, ['{}{}'.format(
                url,
                a.get('href')
            ) for a in self._chapters(url)])]
        return result

    @staticmethod
    def _rebuild_dict_to_tuple(_dict):
        result = []
        for i in _dict:
            result += [(i, [a for a in _dict[i]])]
        return result

    # http://animextremist.com/mangas-online/onepiece-manga/
    def _chapters_without_dirs(self, items):
        result = {}
        r = self.provider.re.compile(r'(.+?-\d+)')  # todo
        for i in items:
            href = i.get('href')
            key = self.provider.re.search(r, href).group(1)
            if result.get(key) is None:
                result[key] = []
            result[key].append('{}{}'.format(self.path, href))
        return self._rebuild_dict_to_tuple(result)

    def get_chapters(self):
        items = self._chapters()
        if len(items) and items[0].get('href').find('.html') < 0:
            items = self._chapters_with_dirs(items)
        else:
            items = self._chapters_without_dirs(items)
        return self.sort_items(items)

    def get_page_image(self, src, selector, attr='src') -> str:
        image = self.provider.html_fromstring(src, selector)
        if image and len(image):
            return image[0].get(attr)



from manga_py.provider import Provider
from .helpers.std import Std


class AllHentaiRu(Provider, Std):

    def get_archive_name(self):
        name = self.re.search('/.+/([^/]+/[^/]+)/?', self.chapter)
        return self.normal_arc_name({'vol': name.group(1).split('/', 2)})

    def get_chapter_index(self):
        name = self.re.search('/.+/(?:vol)?([^/]+/[^/]+)/?', self.chapter)
        return name.group(1).replace('/', '-')

    def get_main_content(self):
        return self._get_content('{}/{}?mature=1&mtr=1')

    def get_manga_name(self) -> str:
        return self._get_name(r'\.ru/([^/]+)')

    def get_chapters(self):
        return self._elements('.expandable .cTable tr > td > a')

    def get_files(self):
        _uri = self.http().normalize_uri(self.chapter)
        content = self.http_get(_uri)
        result = self.re.search(r'var pictures.+?(\[\{.+\}\])', content, self.re.M)
        if not result:
            return []
        content = result.group(1).replace("'", '"')
        content = self.re.sub('(\w*):([^/])', r'"\1":\2', content)
        return [i['url'] for i in self.json.loads(content)]

    def get_cover(self):
        return self._cover_from_content('.picture-fotorama > img')

    def save_file(self, idx=None, callback=None, url=None, in_arc_name=None):
        _path = None
        try:
            _path = super().save_file(idx, callback, url, in_arc_name)
        except AttributeError:
            pass
        if _path is None:
            for i in ['a', 'b', 'c']:
                try:
                    _path, idx, _url = self._save_file_params_helper(url, idx)
                    _url = self.re.sub(r'//\w\.', '//%s.' % i, url)

                    self.http().download_file(_url, _path, idx)
                    callable(callback) and callback()
                    self.after_file_save(_path, idx)

                    self._archive.lazy_add(_path)
                    break
                except AttributeError:
                    pass
        return _path

    def book_meta(self) -> dict:
        # todo meta
        pass


main = AllHentaiRu



from manga_py.provider import Provider
from .helpers.std import Std


class ManhwaCo(Provider, Std):

    def get_chapter_index(self) -> str:
        chapter = self.chapter
        return self.re.search(r'\.co/[^/]+/([^/]+)', chapter).group(1)

    def get_main_content(self):
        return self._get_content('{}/{}')

    def get_manga_name(self) -> str:
        return self._get_name(r'\.co/([^/]+)')

    def get_chapters(self):
        return self._elements('.list-group .list-group-item')

    def get_files(self):
        content = self.http_get(self.chapter)
        return self._images_helper(content, 'img.img-fluid')

    def get_cover(self) -> str:
        return self._cover_from_content('.row > div > img.img-responsive')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = ManhwaCo
